package com.supwisdom.dlpay.api.bean

import com.supwisdom.dlpay.exception.RequestParamCheckException
import com.supwisdom.dlpay.framework.util.MD5
import com.supwisdom.dlpay.framework.util.StringUtil
import com.supwisdom.dlpay.framework.util.TradeErrorCode
import com.supwisdom.dlpay.util.ConstantUtil

open class APIRequestParam {
    open val param_map = mutableMapOf<String, String>()

    open fun checkSign(key: String): Boolean {
        val sign = param_map["sign"]
        if (StringUtil.isEmpty(sign)) return false //未签名

        //判断签名
        val signdata = StringUtil.createLinkString(StringUtil.paraFilter(param_map))
        val md5Sign = MD5.encodeByMD5(signdata + key) //默认MD5
        if (sign.equals(md5Sign, true)) {
            return true
        }
        return false
    }
}

class OpenUserParam : APIRequestParam() {
    var syscode: String = "" //必传
    var uid: String = ""    //必传
    var name: String = ""   //必传
    var sex: String? = ""
    var idtype: String = "" //必传
    var idno: String = ""   //必传
    var mobile: String? = ""
    var tel: String? = ""
    var email: String? = ""
    var address: String? = ""
    var zipcode: String? = ""
    var sign: String = "" //必传

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(syscode)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "子系统注册代码不能为空")
        if (StringUtil.isEmpty(uid)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "用户唯一号不能为空")
        if (StringUtil.isEmpty(name)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "姓名不能为空")
        if (!StringUtil.isEmpty(sex) && !ConstantUtil.SEX_MALE.equals(sex, true) && !ConstantUtil.SEX_FEMALE.equals(sex, true)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "性别非法[male|female]")
        if (StringUtil.isEmpty(idtype) || !ConstantUtil.IDTYPE_DICTS.contains(idtype)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件类型非法")
        if (StringUtil.isEmpty(idno) || !StringUtil.isCharAndNum(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "证件号不能为空，且仅支持数字和字母")
        if (ConstantUtil.IDTYPE_IDENTITY == idtype && !StringUtil.isIdentity(idno)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "身份证格式错误")
        if (!StringUtil.isEmpty(mobile) && !StringUtil.isMobile(mobile)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "手机号格式错误")
        if (!StringUtil.isEmpty(email) && !StringUtil.isEmail(email)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "邮箱格式错误")

        param_map.plus(mapOf("syscode" to syscode, "uid" to uid, "name" to name, "sex" to sex, "idtype" to idtype, "idno" to idno, "mobile" to mobile, "tel" to tel, "email" to email, "address" to address, "zipcode" to zipcode, "sign" to sign))
        return true
    }
}

class QueryUserParam : APIRequestParam() {
    var syscode: String = "" //必传
    var userid: String? = ""  // 用户ID二选一 (两者都传取userid)
    var uid: String? = ""    // 用户ID二选一
    var sign: String = "" //必传

    fun checkParam(): Boolean {
        if (StringUtil.isEmpty(syscode)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "子系统注册代码不能为空")
        if (StringUtil.isEmpty(userid) && StringUtil.isEmpty(uid)) throw RequestParamCheckException(TradeErrorCode.REQUEST_PARAM_ERROR, "用户唯一号不能为空")

        param_map.plus(mapOf("syscode" to syscode, "userid" to userid, "uid" to uid, "sign" to sign))
        return true
    }
}