package com.supwisdom.dlpay.api.controller

import com.supwisdom.dlpay.api.bean.OpenUserParam
import com.supwisdom.dlpay.api.bean.QueryUserParam
import com.supwisdom.dlpay.api.service.UserService
import com.supwisdom.dlpay.exception.RequestParamCheckException
import com.supwisdom.dlpay.exception.TransactionException
import com.supwisdom.dlpay.framework.ResponseBodyBuilder
import com.supwisdom.dlpay.framework.service.SystemUtilService
import com.supwisdom.dlpay.framework.util.TradeErrorCode
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.http.ResponseEntity
import org.springframework.web.bind.annotation.PostMapping
import org.springframework.web.bind.annotation.RequestBody
import org.springframework.web.bind.annotation.RequestMapping
import org.springframework.web.bind.annotation.RestController

@RestController
@RequestMapping("/api/user")
class UserAPIController {
    @Autowired
    private lateinit var useService: UserService
    @Autowired
    private lateinit var systemUtilService: SystemUtilService

    @PostMapping("/open")
    fun openAccount(@RequestBody request: OpenUserParam): ResponseEntity<Any> {
        return try {
            if (request.checkParam() && request.checkSign(systemUtilService.getSubsystemSignKey(request.syscode))) {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(TradeErrorCode.REQUEST_SIGN_ERROR, "参数签名错误"))
            }
            useService.findByThirdUniqueIdenty(request.uid, request.syscode).let {
                if (null != it) {
                    ResponseEntity.ok(ResponseBodyBuilder.create()
                            .fail(TradeErrorCode.REGISTER_USER_EXIST, "改用户唯一号已经注册"))
                }

                useService.registerUser(request).let {
                    if (null != it) {
                        ResponseEntity.ok(ResponseBodyBuilder.create()
                                .data("userid", it.userid)
                                .data("uid", request.uid)
                                .success())
                    }

                    ResponseEntity.ok(ResponseBodyBuilder.create()
                            .fail(TradeErrorCode.BUSINESS_DEAL_ERROR, "用户注册失败"))
                }
            }
        } catch (ex: RequestParamCheckException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .requestException(ex, "请求参数错误"))
        } catch (et: TransactionException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .transException(et, "业务处理错误"))
        }
    }

    @PostMapping("/query")
    fun queryAccount(@RequestBody request: QueryUserParam): ResponseEntity<Any> {
        return try {
            if (request.checkParam() && request.checkSign(systemUtilService.getSubsystemSignKey(request.syscode))) {
                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(TradeErrorCode.REQUEST_SIGN_ERROR, "参数签名错误"))
            }

            useService.findByUseridOrThirdUniqueIdenty(request.userid, request.uid, request.syscode).let {
                if (null != it) {
                    val account = useService.findAccountByUserid(it.userid, null)
                    ResponseEntity.ok(ResponseBodyBuilder.create()
                            .data("userid", it.userid)
                            .data("name", it.name)
                            .data("idtype", it.idtype)
                            .data("idno", it.idno)
                            .data("status", it.status)
                            .data("balance", account.let {
                                if (null == it) {
                                    0.0
                                } else {
                                    it.availbal
                                }
                            })
                            .data("points", 0) //积分
                            .success())
                }

                ResponseEntity.ok(ResponseBodyBuilder.create()
                        .fail(TradeErrorCode.ACCOUNT_NOT_EXISTS, "账户不存在"))
            }
        } catch (ex: RequestParamCheckException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .requestException(ex, "请求参数错误"))
        } catch (et: TransactionException) {
            ResponseEntity.ok(ResponseBodyBuilder.create()
                    .transException(et, "业务处理错误"))
        }
    }


}